import bcrypt
from typing import Optional
from molten import schema, field
from sqlalchemy import relationship, Column, Text, String, Boolean
from casbin_api.db import Base, DBMixin
from casbin_api.schema import Link

BCRYPT_ROUNDS = 11


@schema
class AuthToken:
    token: str = field(response_only=True)


@schema
class User:
    id: int = field(response_only=True)
    createdDate: str = field(response_only=True)
    modifiedDate: str = field(response_only=True)
    email: str
    passwd: str = field(request_only=True)
    href: Link = field(response_only=True)


class UserModel(Base, DBMixin):
    __tablename__ = "user_account"
    email = Column(Text)
    passwd = Column(String(255))
    todos = relationship("TodoModel", back_populates="user")
    is_active = Column(Boolean, default=True)
    is_admin = Column(Boolean, default=False)

    def __init__(self, email, passwd, is_admin=False):
        self.email = email
        self.passwd = bcrypt.hashpw(passwd.encode('utf-8', bcrypt.gensalt(BCRYPT_ROUNDS))).decode('utf-8')
        self.is_admin = is_admin

    def check_password(self, passwd):
        """Checks provided password against saved password hash."""
        return bcrypt.checkpw(passwd.encode('utf-8'), self.passwd.encode('utf-8'))
